@@ -28,6 +28,7 @@
 #include "libgossip-marshal.h"
 #include "gossip-account-manager.h"
 #include "gossip-debug.h"
+#include "gossip-protocol.h"
 #include "gossip-utils.h"
 
 #define DEBUG_DOMAIN "AccountManager"
@@ -55,7 +56,7 @@ struct _GossipAccountManagerPriv {
 
 enum {
 	ACCOUNT_ADDED,
-	ACCOUNT_REMOVED, 
+	ACCOUNT_REMOVED,
 	NEW_DEFAULT,
 	LAST_SIGNAL
 };
@@ -85,7 +86,7 @@ gossip_account_manager_class_init (GossipAccountManagerClass *klass)
 
 	object_class->finalize = account_manager_finalize;
 
-	signals[ACCOUNT_ADDED] = 
+	signals[ACCOUNT_ADDED] =
 		g_signal_new ("account-added",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
@@ -94,26 +95,26 @@ gossip_account_manager_class_init (GossipAccountManagerClass *klass)
 			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
 			      1, GOSSIP_TYPE_ACCOUNT);
-	signals[ACCOUNT_REMOVED] = 
+	signals[ACCOUNT_REMOVED] =
 		g_signal_new ("account-removed",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
-			      0, 
+			      0,
 			      NULL, NULL,
 			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
 			      1, GOSSIP_TYPE_ACCOUNT);
-	signals[NEW_DEFAULT] = 
+	signals[NEW_DEFAULT] =
 		g_signal_new ("new-default",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
-			      0, 
+			      0,
 			      NULL, NULL,
 			      libgossip_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
 			      1, GOSSIP_TYPE_ACCOUNT);
 
-	g_type_class_add_private (object_class, 
+	g_type_class_add_private (object_class,
 				  sizeof (GossipAccountManagerPriv));
 }
 
@@ -126,7 +127,7 @@ static void
 account_manager_finalize (GObject *object)
 {
 	GossipAccountManagerPriv *priv;
-	
+
 	priv = GET_PRIV (object);
 
 	g_list_foreach (priv->accounts, (GFunc) g_object_unref, NULL);
@@ -143,14 +144,14 @@ account_manager_finalize (GObject *object)
 GossipAccountManager *
 gossip_account_manager_new (const gchar *filename)
 {
-	
+
 	GossipAccountManager     *manager;
 	GossipAccountManagerPriv *priv;
 
 	manager = g_object_new (GOSSIP_TYPE_ACCOUNT_MANAGER, NULL);
 
 	priv = GET_PRIV (manager);
-	
+
 	if (filename) {
 		priv->accounts_file_name = g_strdup (filename);
 	}
@@ -173,23 +174,23 @@ gossip_account_manager_add (GossipAccountManager *manager,
 	priv = GET_PRIV (manager);
 
 	/* Don't add more than once */
- 	if (!gossip_account_manager_find (manager, gossip_account_get_name (account))) { 
+	if (!gossip_account_manager_find (manager, gossip_account_get_name (account))) {
 		const gchar       *name;
 		GossipAccountType  type;
 
 		type = gossip_account_get_type (account);
 		name = gossip_account_get_name (account);
 
-		gossip_debug (DEBUG_DOMAIN, "Adding %s account with name:'%s'", 
-			      gossip_account_type_to_string (type), 
+		gossip_debug (DEBUG_DOMAIN, "Adding %s account with name:'%s'",
+			      gossip_account_type_to_string (type),
 			      name);
-		
+
 		priv->accounts = g_list_append (priv->accounts, g_object_ref (account));
 
 		g_signal_emit (manager, signals[ACCOUNT_ADDED], 0, account);
 
 		return TRUE;
-	} 
+	}
 
 	return FALSE;
 }
@@ -199,18 +200,18 @@ gossip_account_manager_remove (GossipAccountManager *manager,
 			       GossipAccount        *account)
 {
 	GossipAccountManagerPriv *priv;
-	
+
 	g_return_if_fail (GOSSIP_IS_ACCOUNT_MANAGER (manager));
 	g_return_if_fail (GOSSIP_IS_ACCOUNT (account));
 
 	priv = GET_PRIV (manager);
 
- 	gossip_debug (DEBUG_DOMAIN, 
-		      "Removing account with name:'%s'",  
+	gossip_debug (DEBUG_DOMAIN,
+		      "Removing account with name:'%s'",
 		      gossip_account_get_name (account));
 
 	priv->accounts = g_list_remove (priv->accounts, account);
-	
+
 	g_signal_emit (manager, signals[ACCOUNT_REMOVED], 0, account);
 
 	g_object_unref (account);
@@ -232,15 +233,15 @@ gossip_account_manager_get_accounts (GossipAccountManager *manager)
 	return accounts;
 }
 
-guint 
+guint
 gossip_account_manager_get_count (GossipAccountManager *manager)
 {
 	GossipAccountManagerPriv *priv;
-	
+
 	g_return_val_if_fail (GOSSIP_IS_ACCOUNT_MANAGER (manager), 0);
-	
+
 	priv = GET_PRIV (manager);
-	
+
 	return g_list_length (priv->accounts);
 }
 
@@ -251,10 +252,10 @@ gossip_account_manager_find (GossipAccountManager *manager,
 {
 	GossipAccountManagerPriv *priv;
 	GList                    *l;
-	
+
 	g_return_val_if_fail (GOSSIP_IS_ACCOUNT_MANAGER (manager), NULL);
 	g_return_val_if_fail (name != NULL, NULL);
-	
+
 	priv = GET_PRIV (manager);
 
 	for (l = priv->accounts; l; l = l->next) {
@@ -270,7 +271,7 @@ gossip_account_manager_find (GossipAccountManager *manager,
 	}
 
 	return NULL;
-}	
+}
 
 GossipAccount *
 gossip_account_manager_find_by_id (GossipAccountManager *manager,
@@ -278,10 +279,10 @@ gossip_account_manager_find_by_id (GossipAccountManager *manager,
 {
 	GossipAccountManagerPriv *priv;
 	GList                    *l;
-	
+
 	g_return_val_if_fail (GOSSIP_IS_ACCOUNT_MANAGER (manager), NULL);
 	g_return_val_if_fail (id != NULL, NULL);
-	
+
 	priv = GET_PRIV (manager);
 
 	for (l = priv->accounts; l; l = l->next) {
@@ -291,13 +292,17 @@ gossip_account_manager_find_by_id (GossipAccountManager *manager,
 		account = l->data;
 		account_id = gossip_account_get_id (account);
 
+		if (!account_id) {
+			continue;
+		}
+
 		if (strcmp (account_id, id) == 0) {
 			return account;
 		}
 	}
 
 	return NULL;
-}	
+}
 
 void
 gossip_account_manager_set_overridden_default (GossipAccountManager *manager,
@@ -310,15 +315,15 @@ gossip_account_manager_set_overridden_default (GossipAccountManager *manager,
 
 	priv = GET_PRIV (manager);
 
- 	gossip_debug (DEBUG_DOMAIN, 
-		      "Setting overriding default account with name:'%s'",  
-		      name); 
+	gossip_debug (DEBUG_DOMAIN,
+		      "Setting overriding default account with name:'%s'",
+		      name);
 
 	g_free (priv->default_name_override);
 	priv->default_name_override = g_strdup (name);
 }
 
-void 
+void
 gossip_account_manager_set_default (GossipAccountManager *manager,
 				    GossipAccount        *account)
 {
@@ -329,10 +334,10 @@ gossip_account_manager_set_default (GossipAccountManager *manager,
 	g_return_if_fail (GOSSIP_IS_ACCOUNT (account));
 
 	priv = GET_PRIV (manager);
-	
- 	gossip_debug (DEBUG_DOMAIN, 
-		      "Setting default account with name:'%s'",  
-		      gossip_account_get_name (account)); 
+
+	gossip_debug (DEBUG_DOMAIN,
+		      "Setting default account with name:'%s'",
+		      gossip_account_get_name (account));
 
 	name = gossip_account_get_name (account);
 	g_return_if_fail (name != NULL);
@@ -399,11 +404,11 @@ gossip_account_manager_store (GossipAccountManager *manager)
 {
 	g_return_val_if_fail (GOSSIP_IS_ACCOUNT_MANAGER (manager), FALSE);
 
- 	gossip_debug (DEBUG_DOMAIN, "Saving accounts");  
-	
+	gossip_debug (DEBUG_DOMAIN, "Saving accounts");
+
 	return account_manager_file_save (manager);
 }
- 
+
 /*
  * API to save/load and parse the accounts file.
  */
@@ -423,7 +428,7 @@ account_manager_get_all (GossipAccountManager *manager)
 		if (!g_file_test (dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) {
 			g_mkdir_with_parents (dir, S_IRUSR | S_IWUSR | S_IXUSR);
 		}
-		
+
 		file_with_path = g_build_filename (dir, ACCOUNTS_XML_FILENAME, NULL);
 		g_free (dir);
 	} else {
@@ -436,7 +441,7 @@ account_manager_get_all (GossipAccountManager *manager)
 		g_free (file_with_path);
 		return FALSE;
 	}
-	
+
 	g_free (file_with_path);
 
 	return TRUE;
@@ -446,16 +451,22 @@ static void
 account_manager_parse_account (GossipAccountManager *manager,
 			       xmlNodePtr            node)
 {
-	GossipAccount *account;
-	xmlNodePtr     child;
-	gchar         *str;
+	GossipAccount  *account;
+	GossipProtocol *protocol;
+	xmlNodePtr      child;
+	gchar          *str;
 
-	/* Default values */
-	account = g_object_new (GOSSIP_TYPE_ACCOUNT,
-				"type", GOSSIP_ACCOUNT_TYPE_JABBER,
-				"auto_connect", TRUE,
-				"port", 5222,
-				NULL);
+	str = xmlGetProp (node, "type");
+	if (str) {
+		protocol = gossip_protocol_new_from_account_type (gossip_account_string_to_type (str));
+	} else {
+		protocol = gossip_protocol_new_from_account_type (GOSSIP_ACCOUNT_TYPE_JABBER);
+	}
+
+	account = gossip_protocol_new_account (protocol);
+
+	g_object_unref (protocol);
+	xmlFree (str);
 
 	child = node->children;
 	while (child) {
@@ -464,9 +475,16 @@ account_manager_parse_account (GossipAccountManager *manager,
 		tag = (gchar *) child->name;
 		str = (gchar *) xmlNodeGetContent (child);
 
+		if (!str) {
+			continue;
+		}
+
 		if (strcmp (tag, "name") == 0) {
 			gossip_account_set_name (account, str);
 		}
+		else if (strcmp (tag, "id") == 0) {
+			gossip_account_set_id (account, str);
+		}
 		else if (strcmp (tag, "auto_connect") == 0) {
 			gossip_account_set_auto_connect (account, strcmp (str, "yes") == 0);
 		}
@@ -480,64 +498,57 @@ account_manager_parse_account (GossipAccountManager *manager,
 				      NULL);
 		}
 		else if (strcmp (tag, "parameter") == 0) {
-			GValue                  *g_value;
 			gchar                   *param_name;
 			gchar                   *type_str;
-			gchar                   *required_str;
-			gchar                   *has_default_str;
 			GType                    type;
-			GossipAccountParamFlags  flags = 0;
+			GValue                  *g_value;
 
 			param_name = xmlGetProp (child, "name");
 			type_str = xmlGetProp (child, "type");
-			required_str = xmlGetProp (child, "required");
-			has_default_str = xmlGetProp (child, "has-default");
-
-			if (strcmp (required_str, "yes") == 0) {
-				flags |= GOSSIP_ACCOUNT_PARAM_FLAG_REQUIRED;
-			}
-			if (strcmp (has_default_str, "yes") == 0) {
-				flags |= GOSSIP_ACCOUNT_PARAM_FLAG_HAS_DEFAULT;
-			}
 
 			type = gossip_dbus_type_to_g_type (type_str);
 			g_value = gossip_string_to_g_value (str, type);
-			
-			gossip_account_param_set_full (account, 
-						       param_name,
-						       g_value,
-						       flags);
+
+			/* Compatibility: The ID was defined here
+			 * in old accounts.xml files. */
+			if (strcmp (param_name, "id") != 0) {
+				gossip_account_param_set_g_value (account,
+								  param_name,
+								  g_value);			
+			} else {
+				/* Save in the new format */
+				need_saving = TRUE;
+				gossip_account_set_id (account, str);
+			}
 
 			g_value_unset (g_value);
 			g_free (g_value);
 			xmlFree (param_name);
 			xmlFree (type_str);
-			xmlFree (required_str);
-			xmlFree (has_default_str);
 		}
 		/* Those are deprecated and kept for compatibility only */
-		else if (strcmp (tag, "id") == 0) {
-			gossip_account_set_id (account, str);
-		}
 		else if (strcmp (tag, "resource") == 0) {
-			gossip_account_set_resource (account, str);
+			gossip_account_param_set (account, "resource", str, NULL);
 		}
 		else if (strcmp (tag, "password") == 0) {
-			gossip_account_set_password (account, str);
+			gossip_account_param_set (account, "password", str, NULL);
 		}
 		else if (strcmp (tag, "server") == 0) {
-			gossip_account_set_server (account, str);
+			gossip_account_param_set (account, "server", str, NULL);
 		}
 		else if (strcmp (tag, "port") == 0) {
 			guint tmp_port;
-			
+
 			tmp_port = atoi (str);
 			if (tmp_port != 0) {
-				gossip_account_set_port (account, tmp_port);
+				gossip_account_param_set (account, "port",
+							  tmp_port, NULL);
 			}
 		}
 		else if (strcmp (tag, "use_ssl") == 0) {
-			gossip_account_set_use_ssl (account, strcmp (str, "yes") == 0);
+			gossip_account_param_set (account, "use_ssl",
+						  strcmp (str, "yes") == 0,
+						  NULL);
 		}
 
 		xmlFree (str);
@@ -546,8 +557,10 @@ account_manager_parse_account (GossipAccountManager *manager,
 	}
 #ifdef RESOURCE_HACK
 	const gchar *resource_found = NULL;
+	const gchar *id;
 
-	str = g_strdup (gossip_account_get_id (account));
+	id = gossip_account_get_id (account);
+	str = g_strdup (id);
 	if (str) {
 		/* FIXME: This hack is so we don't get bug
 		 * reports and basically gets the resource
@@ -560,14 +573,16 @@ account_manager_parse_account (GossipAccountManager *manager,
 			resource_found = (ch + 1);
 			str[ch - str] = '\0';
 
-			g_printerr ("Converting ID... (id:'%s', resource:'%s')\n", 
+			g_printerr ("Converting ID... (id:'%s', resource:'%s')\n",
 				    str, resource_found);
 		}
 	}
 
 	if (resource_found) {
-		gossip_account_set_id (account, str);
-		gossip_account_set_resource (account, resource_found);
+		gossip_account_param_set (account,
+					  "id", str,
+					  "resource", resource_found,
+					  NULL);
 		need_saving = TRUE;
 	}
 
@@ -580,23 +595,23 @@ account_manager_parse_account (GossipAccountManager *manager,
 }
 
 static gboolean
-account_manager_file_parse (GossipAccountManager *manager, 
-			    const gchar          *filename) 
+account_manager_file_parse (GossipAccountManager *manager,
+			    const gchar          *filename)
 {
 	GossipAccountManagerPriv *priv;
- 	xmlParserCtxtPtr          ctxt;
+	xmlParserCtxtPtr          ctxt;
 	xmlDocPtr                 doc;
 	xmlNodePtr                accounts;
 	xmlNodePtr                node;
 	gchar                    *str;
 
 	priv = GET_PRIV (manager);
-	
-	gossip_debug (DEBUG_DOMAIN, 
-		      "Attempting to parse file:'%s'...", 
+
+	gossip_debug (DEBUG_DOMAIN,
+		      "Attempting to parse file:'%s'...",
 		      filename);
 
- 	ctxt = xmlNewParserCtxt ();
+	ctxt = xmlNewParserCtxt ();
 
 	/* Parse and validate the file. */
 	doc = xmlCtxtReadFile (ctxt, filename, NULL, 0);
@@ -615,7 +630,7 @@ account_manager_file_parse (GossipAccountManager *manager,
 
 	/* The root node, accounts. */
 	accounts = xmlDocGetRootElement (doc);
-	
+
 	node = accounts->children;
 	while (node) {
 		if (strcmp ((gchar *) node->name, "default") == 0) {
@@ -633,15 +648,15 @@ account_manager_file_parse (GossipAccountManager *manager,
 
 		node = node->next;
 	}
-	
-	gossip_debug (DEBUG_DOMAIN, 
-		      "Parsed %d accounts", 
+
+	gossip_debug (DEBUG_DOMAIN,
+		      "Parsed %d accounts",
 		      g_list_length (priv->accounts));
 
-	gossip_debug (DEBUG_DOMAIN, 
-		      "Default account is:'%s'", 
+	gossip_debug (DEBUG_DOMAIN,
+		      "Default account is:'%s'",
 		      priv->default_name);
-	
+
 	xmlFreeDoc(doc);
 	xmlFreeParserCtxt (ctxt);
 
@@ -659,7 +674,7 @@ static gboolean
 account_manager_file_save (GossipAccountManager *manager)
 {
 	GossipAccountManagerPriv *priv;
-	xmlDocPtr                 doc;  
+	xmlDocPtr                 doc;
 	xmlNodePtr                root;
 	GList                    *accounts;
 	GList                    *l;
@@ -668,13 +683,13 @@ account_manager_file_save (GossipAccountManager *manager)
 	mode_t                    old_mask;
 
 	priv = GET_PRIV (manager);
-	
+
 	if (priv->accounts_file_name) {
 		xml_file = g_strdup (priv->accounts_file_name);
 	} else {
 		xml_dir = g_build_filename (g_get_home_dir (), ".gnome2", PACKAGE_NAME, NULL);
 		g_mkdir_with_parents (xml_dir, S_IRUSR | S_IWUSR | S_IXUSR);
-					 
+
 		xml_file = g_build_filename (xml_dir, ACCOUNTS_XML_FILENAME, NULL);
 		g_free (xml_dir);
 	}
@@ -686,9 +701,9 @@ account_manager_file_save (GossipAccountManager *manager)
 	if (!priv->default_name) {
 		priv->default_name = g_strdup ("Default");
 	}
-	
-	xmlNewChild (root, NULL, 
-		    "default", 
+
+	xmlNewChild (root, NULL,
+		    "default",
 		    priv->default_name);
 
 	accounts = gossip_account_manager_get_accounts (manager);
@@ -703,8 +718,9 @@ account_manager_file_save (GossipAccountManager *manager)
 		type = gossip_account_type_to_string (gossip_account_get_type (account));
 
 		node = xmlNewChild (root, NULL, "account", NULL);
-		xmlNewTextChild (node, NULL, "type", type);
+		xmlNewProp (node, "type", type);
 		xmlNewTextChild (node, NULL, "name", gossip_account_get_name (account));
+		xmlNewTextChild (node, NULL, "id", gossip_account_get_id (account));
 		xmlNewTextChild (node, NULL, "auto_connect", gossip_account_get_auto_connect (account) ? "yes" : "no");
 		xmlNewTextChild (node, NULL, "use_proxy", gossip_account_get_use_proxy (account) ? "yes" : "no");
 
@@ -732,7 +748,7 @@ account_manager_file_save (GossipAccountManager *manager)
 
 	g_list_foreach (accounts, (GFunc) g_object_unref, NULL);
 	g_list_free (accounts);
-	
+
 	return TRUE;
 }
 
@@ -744,19 +760,13 @@ account_manager_add_parameters_foreach (GossipAccount      *account,
 {
 	gchar       *value_str;
 	const gchar *type_str;
-	gboolean     required;
-	gboolean     has_default;
 
 	value_str = gossip_g_value_to_string (&param->g_value);
 	type_str = gossip_g_type_to_dbus_type (G_VALUE_TYPE (&param->g_value));
-	required = param->flags & GOSSIP_ACCOUNT_PARAM_FLAG_REQUIRED;
-	has_default = param->flags & GOSSIP_ACCOUNT_PARAM_FLAG_HAS_DEFAULT;
 
 	node = xmlNewTextChild (node, NULL, "parameter", value_str);
 	xmlNewProp (node, "name", param_name);
 	xmlNewProp (node, "type", type_str);
-	xmlNewProp (node, "required", required ? "yes" : "no");
-	xmlNewProp (node, "has-default", has_default ? "yes" : "no");
 
 	g_free (value_str);
 }
