@@ -47,8 +47,13 @@ typedef enum {
 
 typedef void (*GossipChatroomJoinCb) (GossipChatroomProvider   *provider,
 				      GossipChatroomJoinResult  result,
-                                      gint                      id,
+				      gint                      id,
 				      gpointer                  user_data);
+typedef void (*GossipChatroomBrowseCb) (GossipChatroomProvider   *provider,
+					const gchar              *server,
+					GList                    *rooms,
+					GError                   *error,
+					gpointer                  user_data);
 
 struct _GossipChatroomProviderIface {
 	GTypeInterface g_iface;
@@ -71,8 +76,10 @@ struct _GossipChatroomProviderIface {
 					     const gchar            *new_nick);
 	void             (*leave)           (GossipChatroomProvider *provider,
 					     GossipChatroomId        id);
-	GossipChatroom * (*find)            (GossipChatroomProvider *provider,
+	GossipChatroom * (*find_by_id)      (GossipChatroomProvider *provider,
 					     GossipChatroomId        id);
+	GossipChatroom * (*find)            (GossipChatroomProvider *provider,
+					     GossipChatroom         *chatroom);
 	void             (*invite)          (GossipChatroomProvider *provider,
 					     GossipChatroomId        id,
 					     GossipContact          *contact,
@@ -85,12 +92,16 @@ struct _GossipChatroomProviderIface {
 					     GossipChatroomInvite   *invite,
 					     const gchar            *reason);
 	GList *          (*get_rooms)       (GossipChatroomProvider *provider);
+	void             (*browse_rooms)    (GossipChatroomProvider *provider,
+					     const gchar            *server,
+					     GossipChatroomBrowseCb  callback,
+					     gpointer                user_data);
 };
 
 GType        gossip_chatroom_provider_get_type           (void) G_GNUC_CONST;
 
 GossipChatroomId
-             gossip_chatroom_provider_join               (GossipChatroomProvider *provider,
+	     gossip_chatroom_provider_join               (GossipChatroomProvider *provider,
 							  GossipChatroom         *chatroom,
 							  GossipChatroomJoinCb    callback,
 							  gpointer                user_data);
@@ -108,8 +119,11 @@ void         gossip_chatroom_provider_change_nick        (GossipChatroomProvider
 void         gossip_chatroom_provider_leave              (GossipChatroomProvider *provider,
 							  GossipChatroomId        id);
 GossipChatroom *
-             gossip_chatroom_provider_find               (GossipChatroomProvider *provider,
+	     gossip_chatroom_provider_find_by_id         (GossipChatroomProvider *provider,
 							  GossipChatroomId        id);
+GossipChatroom *
+	     gossip_chatroom_provider_find               (GossipChatroomProvider *provider,
+							  GossipChatroom         *chatroom);
 void         gossip_chatroom_provider_invite             (GossipChatroomProvider *provider,
 							  GossipChatroomId        id,
 							  GossipContact          *contact,
@@ -123,6 +137,10 @@ void         gossip_chatroom_provider_invite_decline     (GossipChatroomProvider
 							  const gchar            *reason);
 
 GList *      gossip_chatroom_provider_get_rooms          (GossipChatroomProvider *provider);
+void         gossip_chatroom_provider_browse_rooms       (GossipChatroomProvider *provider,
+							  const gchar            *server,
+							  GossipChatroomBrowseCb  callback,
+							  gpointer                user_data);
 
 const gchar *gossip_chatroom_provider_join_result_as_str (GossipChatroomJoinResult result);
 
