@@ -23,11 +23,9 @@
 #include <glib/gi18n.h>
 #include <string.h>
 
-#include "gossip-utils.h"
+#include "gossip-account.h"
 #include "gossip-contact.h"
-
-#define DEBUG_MSG(x) 
-/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n"); */
+#include "gossip-utils.h"
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_CONTACT, GossipContactPriv))
 
@@ -134,7 +132,7 @@ contact_class_init (GossipContactClass *class)
 							      "The name of the contact",
 							      NULL,
 							      G_PARAM_READWRITE));
-	
+
 	g_object_class_install_property (object_class,
 					 PROP_ID,
 					 g_param_spec_string ("id",
@@ -142,7 +140,7 @@ contact_class_init (GossipContactClass *class)
 							      "String identifying contact",
 							      NULL,
 							      G_PARAM_READWRITE));
-	
+
 	g_object_class_install_property (object_class,
 					 PROP_ACTIVE_PRESENCE,
 					 g_param_spec_object ("active-presence",
@@ -166,7 +164,7 @@ contact_class_init (GossipContactClass *class)
 					 g_param_spec_pointer ("avatar",
 							       "Avatar image",
 							       "The avatar image",
-							       G_PARAM_READWRITE));						   
+							       G_PARAM_READWRITE));
 	g_object_class_install_property (object_class,
 					 PROP_AVATAR_SIZE,
 					 g_param_spec_uint ("avatar_size",
@@ -194,7 +192,7 @@ contact_init (GossipContact *contact)
 	GossipContactPriv *priv;
 
 	priv = GET_PRIV (contact);
-	
+
 	priv->type        = GOSSIP_CONTACT_TYPE_TEMPORARY;
 	priv->name        = NULL;
 	priv->id          = NULL;
@@ -211,33 +209,23 @@ contact_finalize (GObject *object)
 	GossipContactPriv *priv;
 
 	priv = GET_PRIV (object);
-	
+
 	g_free (priv->name);
 	g_free (priv->id);
 	g_free (priv->avatar);
 
 	if (priv->presences) {
-		GList *l;
-
-		for (l = priv->presences; l; l = l->next) {
-			g_object_unref (l->data);
-		}
-
+		g_list_foreach (priv->presences, (GFunc) g_object_unref, NULL);
 		g_list_free (priv->presences);
 	}
 
 	if (priv->groups) {
-		GList *l;
-
-		for (l = priv->groups; l; l = l->next) {
-			g_free (l->data);
-		}
-
+		g_list_foreach (priv->groups, (GFunc) g_free, NULL);
 		g_list_free (priv->groups);
 	}
-	
+
 	g_object_unref (priv->account);
-	
+
 	(G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
@@ -275,7 +263,7 @@ contact_get_property (GObject    *object,
 		break;
 	case PROP_AVATAR:
 		g_value_set_pointer (value, priv->avatar);
-		break;		
+		break;
 	case PROP_AVATAR_SIZE:
 		g_value_set_uint (value, priv->avatar_size);
 		break;
@@ -288,7 +276,7 @@ contact_get_property (GObject    *object,
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
-                break;
+		break;
 	};
 }
 
@@ -311,25 +299,25 @@ contact_set_property (GObject      *object,
 					 g_value_get_string (value));
 		break;
 	case PROP_ID:
-		gossip_contact_set_id (GOSSIP_CONTACT (object), 
+		gossip_contact_set_id (GOSSIP_CONTACT (object),
 				       g_value_get_string (value));
 		break;
 	case PROP_SUBSCRIPTION:
 		priv->subscription = g_value_get_int (value);
 		break;
 	case PROP_AVATAR:
- 		priv->avatar = g_value_get_pointer (value); 
-		break;		
+		priv->avatar = g_value_get_pointer (value);
+		break;
 	case PROP_AVATAR_SIZE:
 		priv->avatar_size = g_value_get_uint (value);
 		break;
 	case PROP_ACCOUNT:
-		gossip_contact_set_account (GOSSIP_CONTACT (object), 
+		gossip_contact_set_account (GOSSIP_CONTACT (object),
 					    GOSSIP_ACCOUNT (g_value_get_object (value)));
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
-                break;
+		break;
 	};
 }
 
@@ -337,8 +325,8 @@ GossipContact *
 gossip_contact_new (GossipContactType  type,
 		    GossipAccount     *account)
 {
-	return g_object_new (GOSSIP_TYPE_CONTACT, 
-			     "type", type, 
+	return g_object_new (GOSSIP_TYPE_CONTACT,
+			     "type", type,
 			     "account", account,
 			     NULL);
 }
@@ -349,15 +337,15 @@ gossip_contact_new_full (GossipContactType  type,
 			 const gchar       *id,
 			 const gchar       *name)
 {
-	return g_object_new (GOSSIP_TYPE_CONTACT, 
-			     "type", type, 
+	return g_object_new (GOSSIP_TYPE_CONTACT,
+			     "type", type,
 			     "account", account,
 			     "name", name,
 			     "id", id,
 			     NULL);
 }
 
-GossipContact *    
+GossipContact *
 gossip_contact_copy (GossipContact *contact)
 {
 	GossipContact     *new_contact;
@@ -366,27 +354,31 @@ gossip_contact_copy (GossipContact *contact)
 	GList             *l;
 
 	g_return_val_if_fail (GOSSIP_IS_CONTACT (contact), NULL);
-	
+
 	priv = GET_PRIV (contact);
 
-	new_contact = gossip_contact_new_full (priv->type, 
+	new_contact = gossip_contact_new_full (priv->type,
 					       priv->account,
 					       priv->id,
 					       priv->name);
 
 	new_priv = GET_PRIV (new_contact);
-	
+
 	new_priv->subscription = priv->subscription;
-	
+
 	for (l = priv->groups; l; l = l->next) {
-		new_priv->groups = g_list_append (new_priv->groups, 
-						  g_strdup (l->data));
+		new_priv->groups = g_list_prepend (new_priv->groups,
+						   g_strdup (l->data));
 	}
 
+	priv->groups = g_list_reverse (priv->groups);
+
 	for (l = priv->presences; l; l = l->next) {
-		new_priv->presences = g_list_append (new_priv->presences, 
-						     g_strdup (l->data));
+		new_priv->presences = g_list_prepend (new_priv->presences,
+						      g_strdup (l->data));
 	}
+	
+	priv->presences = g_list_reverse (priv->presences);
 
 	return new_contact;
 }
@@ -396,7 +388,7 @@ gossip_contact_get_type (GossipContact *contact)
 {
 	GossipContactPriv *priv;
 
-	g_return_val_if_fail (GOSSIP_IS_CONTACT (contact), 
+	g_return_val_if_fail (GOSSIP_IS_CONTACT (contact),
 			      GOSSIP_CONTACT_TYPE_TEMPORARY);
 
 	priv = GET_PRIV (contact);
@@ -437,7 +429,7 @@ gossip_contact_get_name (GossipContact *contact)
 }
 
 const guchar *
-gossip_contact_get_avatar (GossipContact *contact, 
+gossip_contact_get_avatar (GossipContact *contact,
 			   gsize         *avatar_size)
 {
 	GossipContactPriv *priv;
@@ -445,12 +437,12 @@ gossip_contact_get_avatar (GossipContact *contact,
 
 	g_return_val_if_fail (GOSSIP_IS_CONTACT (contact), NULL);
 	g_return_val_if_fail (avatar_size != NULL, NULL);
-	
+
 	priv = GET_PRIV (contact);
 
 	avatar = (guchar*) priv->avatar;
 	*avatar_size = priv->avatar_size;
-	
+
 	return avatar;
 }
 
@@ -489,7 +481,7 @@ gossip_contact_get_presence_for_resource (GossipContact *contact,
 {
 	GossipContactPriv *priv;
 	GList             *l;
-	
+
 	g_return_val_if_fail (GOSSIP_IS_CONTACT (contact), NULL);
 	g_return_val_if_fail (resource != NULL, NULL);
 
@@ -507,7 +499,7 @@ gossip_contact_get_presence_for_resource (GossipContact *contact,
 	return NULL;
 }
 
-GList *     
+GList *
 gossip_contact_get_presence_list (GossipContact *contact)
 {
 	GossipContactPriv *priv;
@@ -562,7 +554,7 @@ gossip_contact_set_id (GossipContact *contact,
 }
 
 void
-gossip_contact_set_name (GossipContact *contact, 
+gossip_contact_set_name (GossipContact *contact,
 			 const gchar   *name)
 {
 	GossipContactPriv *priv;
@@ -575,11 +567,11 @@ gossip_contact_set_name (GossipContact *contact,
 	g_free (priv->name);
 	priv->name = g_strdup (name);
 
-	g_object_notify (G_OBJECT (contact), "name"); 
+	g_object_notify (G_OBJECT (contact), "name");
 }
 
-void 
-gossip_contact_set_avatar (GossipContact *contact, 
+void
+gossip_contact_set_avatar (GossipContact *contact,
 			   const guchar  *avatar,
 			   gsize          avatar_size)
 {
@@ -588,7 +580,7 @@ gossip_contact_set_avatar (GossipContact *contact,
 	g_return_if_fail (GOSSIP_IS_CONTACT (contact));
 
 	priv = GET_PRIV (contact);
-	
+
 	g_free (priv->avatar);
 
 	if (avatar) {
@@ -599,8 +591,8 @@ gossip_contact_set_avatar (GossipContact *contact,
 		priv->avatar_size = 0;
 	}
 
-	g_object_notify (G_OBJECT (contact), "avatar"); 
-	g_object_notify (G_OBJECT (contact), "avatar-size"); 
+	g_object_notify (G_OBJECT (contact), "avatar");
+	g_object_notify (G_OBJECT (contact), "avatar-size");
 }
 
 void
@@ -617,56 +609,68 @@ gossip_contact_set_account (GossipContact *contact,
 	if (priv->account) {
 		g_object_unref (priv->account);
 	}
-	
+
 	if (account) {
 		priv->account = g_object_ref (account);
 	} else {
 		priv->account = NULL;
 	}
 
-	g_object_notify (G_OBJECT (contact), "account"); 
+	g_object_notify (G_OBJECT (contact), "account");
 }
 
-void 
-gossip_contact_add_presence (GossipContact  *contact, 
+void
+gossip_contact_add_presence (GossipContact  *contact,
 			     GossipPresence *presence)
 {
 	GossipContactPriv *priv;
+	GossipPresence    *this_presence;
 	GList             *l;
-	gboolean           old;
 
 	g_return_if_fail (GOSSIP_IS_CONTACT (contact));
 	g_return_if_fail (GOSSIP_IS_PRESENCE (presence));
 
 	priv = GET_PRIV (contact);
 
-	old = FALSE;
+	for (l = priv->presences; l && presence; l = l->next) {
+		this_presence = l->data;
 
-	for (l = priv->presences; l; l = l->next) {
-		if (gossip_presence_resource_equal (l->data, presence)) {
-			/* Replace the old presence with the new */
-			g_object_unref (l->data);
-			l->data = g_object_ref (presence);
-			old = TRUE;
+		if (gossip_presence_resource_equal (this_presence, presence)) {
+			gint ref_count;
+
+			ref_count = G_OBJECT (presence)->ref_count;
+
+			/* Remove old presence for this resource, we
+			 * would use g_list_remove_all() here but we
+			 * want to make sure we unref for each
+			 * instance we find it in the list.
+			 */
+			priv->presences = g_list_remove (priv->presences, this_presence);
+			g_object_unref (this_presence);
+
+			if (!priv->presences || ref_count <= 1) {
+				break;
+			}
+
+			/* Reset list to beginnging to make sure we
+			 * didn't miss any duplicates.
+			 */
+			l = priv->presences;
 		}
 	}
 
-	if (!old) {
-		priv->presences = g_list_prepend (priv->presences, 
-						  g_object_ref (presence));
-	}
-
-	priv->presences = g_list_sort (priv->presences,
-				       gossip_presence_priority_sort_func);
-	/* Highest priority first */
-	priv->presences = g_list_reverse (priv->presences);
+	/* Add new presence */
+	priv->presences = g_list_insert_sorted (priv->presences, 
+						g_object_ref (presence),
+						gossip_presence_sort_func);
 }
 
-void 
-gossip_contact_remove_presence (GossipContact  *contact, 
+void
+gossip_contact_remove_presence (GossipContact  *contact,
 				GossipPresence *presence)
 {
 	GossipContactPriv *priv;
+	GossipPresence    *this_presence;
 	GList             *l;
 
 	g_return_if_fail (GOSSIP_IS_CONTACT (contact));
@@ -675,43 +679,55 @@ gossip_contact_remove_presence (GossipContact  *contact,
 	priv = GET_PRIV (contact);
 
 	for (l = priv->presences; l; l = l->next) {
-		if (gossip_presence_resource_equal (l->data, presence)) {
-			break;
+		this_presence = l->data;
+
+		if (gossip_presence_resource_equal (this_presence, presence)) {
+			gint ref_count;
+
+			ref_count = G_OBJECT (presence)->ref_count;
+
+			/* Remove old presence for this resource, we
+			 * would use g_list_remove_all() here but we
+			 * want to make sure we unref for each
+			 * instance we find it in the list.
+			 */
+			priv->presences = g_list_remove (priv->presences, this_presence);
+			g_object_unref (this_presence);
+
+			if (!priv->presences || ref_count <= 1) {
+				break;
+			}
+
+			/* Reset list to beginnging to make sure we
+			 * didn't miss any duplicates.
+			 */
+			l = priv->presences;
 		}
 	}
 
-	if (l) {
-		priv->presences = g_list_remove_link (priv->presences, l);
-		
-		g_object_unref (l->data);
-		g_list_free_1 (l);
-	}
-
 	priv->presences = g_list_sort (priv->presences,
-				       gossip_presence_priority_sort_func);
-	/* Highest priority first */
-	priv->presences = g_list_reverse (priv->presences);
+				       gossip_presence_sort_func);
 }
 
 gboolean
-gossip_contact_set_groups (GossipContact *contact, 
+gossip_contact_set_groups (GossipContact *contact,
 			   GList         *groups)
 {
 	GossipContactPriv *priv;
 	GList             *old_groups, *l;
 
 	g_return_val_if_fail (GOSSIP_IS_CONTACT (contact), FALSE);
-	
+
 	priv = GET_PRIV (contact);
 
 	old_groups = priv->groups;
 	priv->groups = NULL;
 
 	for (l = groups; l; l = l->next) {
-		priv->groups = g_list_append (priv->groups, 
+		priv->groups = g_list_append (priv->groups,
 					      g_strdup (l->data));
 	}
-	
+
 	g_list_foreach (old_groups, (GFunc) g_free, NULL);
 	g_list_free (old_groups);
 
@@ -733,39 +749,18 @@ gossip_contact_set_subscription (GossipContact      *contact,
 	g_object_notify (G_OBJECT (contact), "subscription");
 }
 
-gint
-gossip_contact_compare (gconstpointer a,
-			gconstpointer b)
-{
-	const gchar *id_a;
-	const gchar *id_b;
-
-	g_return_val_if_fail (GOSSIP_IS_CONTACT (a), 0);
-	g_return_val_if_fail (GOSSIP_IS_CONTACT (b), 0);
-
-	id_a = gossip_contact_get_id (GOSSIP_CONTACT (a));
-	id_b = gossip_contact_get_id (GOSSIP_CONTACT (b));
-
-	/* FIXME: Maybe use utf8 strcmp? At least jabber ids are UTF8. */
-	return strcmp (id_a, id_b);
-}
-
 gint
 gossip_contact_name_compare (gconstpointer a,
 			     gconstpointer b)
 {
 	g_return_val_if_fail (GOSSIP_IS_CONTACT (a), 0);
 	g_return_val_if_fail (GOSSIP_IS_CONTACT (b), 0);
-	
-	DEBUG_MSG (("Contact: Comparing %s vs. %s",
-		    gossip_contact_get_name (GOSSIP_CONTACT (a)),
-		    gossip_contact_get_name (GOSSIP_CONTACT (b))));
 
 	return strcmp (gossip_contact_get_name (GOSSIP_CONTACT (a)),
 		       gossip_contact_get_name (GOSSIP_CONTACT (b)));
 }
 
-gint           
+gint
 gossip_contact_name_case_compare (gconstpointer a,
 				  gconstpointer b)
 {
@@ -775,8 +770,8 @@ gossip_contact_name_case_compare (gconstpointer a,
 	return gossip_contact_name_case_n_compare (a, b, -1);
 }
 
-gint           
-gossip_contact_name_case_n_compare (gconstpointer a, 
+gint
+gossip_contact_name_case_n_compare (gconstpointer a,
 				    gconstpointer b,
 				    gsize         n)
 {
@@ -787,7 +782,7 @@ gossip_contact_name_case_n_compare (gconstpointer a,
 
 	name_a = gossip_contact_get_name (GOSSIP_CONTACT (a));
 	name_b = gossip_contact_get_name (GOSSIP_CONTACT (b));
-					  
+
 	return gossip_strncasecmp (name_a, name_b, -1);
 }
 
@@ -795,24 +790,35 @@ gboolean
 gossip_contact_equal (gconstpointer v1,
 		      gconstpointer v2)
 {
-	const gchar *id_a;
-	const gchar *id_b;
+	const gchar   *id_a;
+	const gchar   *id_b;
+	GossipAccount *account_a;
+	GossipAccount *account_b;
 
 	g_return_val_if_fail (GOSSIP_IS_CONTACT (v1), FALSE);
 	g_return_val_if_fail (GOSSIP_IS_CONTACT (v2), FALSE);
-	
+
 	id_a = gossip_contact_get_id (GOSSIP_CONTACT (v1));
 	id_b = gossip_contact_get_id (GOSSIP_CONTACT (v2));
 
-	return g_str_equal (id_a, id_b);
+	account_a = gossip_contact_get_account (GOSSIP_CONTACT (v1));
+	account_b = gossip_contact_get_account (GOSSIP_CONTACT (v2));
+
+	return g_str_equal (id_a, id_b) &&
+	       gossip_account_equal (account_a, account_b);
 }
 
 guint
 gossip_contact_hash (gconstpointer key)
 {
+	GossipContactPriv *priv;
+
 	g_return_val_if_fail (GOSSIP_IS_CONTACT (key), +1);
 
-	return g_str_hash (gossip_contact_get_id (GOSSIP_CONTACT (key)));
+	priv = GET_PRIV (GOSSIP_CONTACT (key));
+
+	return g_str_hash (gossip_contact_get_id (GOSSIP_CONTACT (key))) + 
+	       gossip_account_hash (gossip_contact_get_account (GOSSIP_CONTACT (key)));
 }
 
 /* convenience functions */
@@ -829,7 +835,7 @@ gossip_contact_is_online (GossipContact *contact)
 	if (priv->presences == NULL) {
 		return FALSE;
 	}
-	
+
 	return TRUE;
 }
 
