@@ -25,7 +25,8 @@
 #include <libxml/tree.h>
 
 #include "gossip-account.h"
-#include "gossip-presence.h"
+
+#define G_STR_EMPTY(x) ((x) == NULL || (x)[0] == '\0')
 
 typedef enum {
 	GOSSIP_REGEX_AS_IS,
@@ -35,31 +36,31 @@ typedef enum {
 	GOSSIP_REGEX_ALL,
 } GossipRegExType;
 
-typedef struct {
-	GossipPresenceState  state;
-	gchar               *string;
-} GossipStatusEntry;
+/* Regular expressions */
+gchar *       gossip_substring           (const gchar     *str,
+					  gint             start,
+					  gint             end);
+gint          gossip_regex_match         (GossipRegExType  type,
+					  const gchar     *msg,
+					  GArray          *start,
+					  GArray          *end);
 
-GList *  gossip_utils_get_status_messages                 (void);
-void     gossip_utils_set_status_messages                 (GList           *list);
-void     gossip_utils_free_status_messages                (GList           *list);
+/* Strings */
+gint          gossip_strcasecmp          (const gchar     *s1,
+					  const gchar     *s2);
+gint          gossip_strncasecmp         (const gchar     *s1,
+					  const gchar     *s2,
+					  gsize            n);
 
-gchar *  gossip_utils_substring                           (const gchar     *str,
-							   gint             start,
-							   gint             end);
-gint     gossip_utils_regex_match                         (GossipRegExType  type,
-							   const gchar     *msg,
-							   GArray          *start,
-							   GArray          *end);
+/* XML */
+gboolean      gossip_xml_validate        (xmlDoc          *doc,
+					  const gchar     *dtd_filename);
 
-GossipPresenceState 
-         gossip_utils_get_presence_state_from_show_string (const gchar     *str); 
-gint     gossip_utils_str_case_cmp                        (const gchar     *s1,
-							   const gchar     *s2);
-gint     gossip_utils_str_n_case_cmp                      (const gchar     *s1,
-							   const gchar     *s2,
-							   gsize            n);
-gboolean gossip_utils_xml_validate                        (xmlDoc          *doc,
-							   const gchar     *dtd_filename);
+/* GValue/GType */
+GType         gossip_dbus_type_to_g_type (const gchar     *dbus_type_string);
+const gchar * gossip_g_type_to_dbus_type (GType            g_type);
+gchar *       gossip_g_value_to_string   (const GValue    *value);
+GValue *      gossip_string_to_g_value   (const gchar     *str,
+					  GType            type);
 
 #endif /*  __GOSSIP_UTILS_H__ */
