@@ -166,6 +166,13 @@ static void             jabber_register_account             (GossipProtocol
 							     gpointer                 user_data);
 static void             jabber_register_cancel              (GossipProtocol          *protocol);
 static GossipAccount *  jabber_new_account                  (GossipProtocol          *protocol);
+static void             jabber_get_avatar_requirements      (GossipProtocol          *protocol,
+							     guint                   *min_width,
+							     guint                   *min_height,
+							     guint                   *max_width,
+							     guint                   *max_height,
+							     gsize                   *max_size,
+							     gchar                  **format);
 static void             jabber_register_connection_open_cb  (LmConnection            *connection,
 							     gboolean                 result,
 							     RegisterData            *ra);
@@ -293,8 +300,10 @@ static void             jabber_chatroom_change_nick         (GossipChatroomProvi
 							     const gchar             *new_nick);
 static void             jabber_chatroom_leave               (GossipChatroomProvider  *provider,
 							     GossipChatroomId         id);
-static GossipChatroom * jabber_chatroom_find                (GossipChatroomProvider  *provider,
+static GossipChatroom * jabber_chatroom_find_by_id          (GossipChatroomProvider  *provider,
 							     GossipChatroomId         id);
+static GossipChatroom * jabber_chatroom_find                (GossipChatroomProvider  *provider,
+							     GossipChatroom          *chatroom);
 static void             jabber_chatroom_invite              (GossipChatroomProvider  *provider,
 							     GossipChatroomId         id,
 							     GossipContact           *contact,
@@ -307,6 +316,10 @@ static void             jabber_chatroom_invite_decline      (GossipChatroomProvi
 							     GossipChatroomInvite    *invite,
 							     const gchar             *reason);
 static GList *          jabber_chatroom_get_rooms           (GossipChatroomProvider  *provider);
+static void             jabber_chatroom_browse_rooms        (GossipChatroomProvider  *provider,
+							     const gchar             *server,
+							     GossipChatroomBrowseCb   callback,
+							     gpointer                 user_data);
 
 /* fts */
 static void             jabber_ft_init                      (GossipFTProviderIface   *iface);
@@ -376,6 +389,7 @@ gossip_jabber_class_init (GossipJabberClass *klass)
 	protocol_class->register_account      = jabber_register_account;
 	protocol_class->register_cancel       = jabber_register_cancel;
 	protocol_class->new_account           = jabber_new_account;
+	protocol_class->get_avatar_requirements = jabber_get_avatar_requirements;
 
 	g_type_class_add_private (object_class, sizeof (GossipJabberPriv));
 }
@@ -477,8 +491,9 @@ jabber_setup (GossipProtocol *protocol,
 	priv->contact = gossip_contact_new (GOSSIP_CONTACT_TYPE_USER,
 					    priv->account);
 
+	id = gossip_account_get_id (account);
+
 	gossip_account_param_get (account,
-				  "id", &id,
 				  "server", &server,
 				  "port", &port,
 				  NULL);
@@ -786,9 +801,9 @@ jabber_connection_open_cb (LmConnection *connection,
 	account = priv->account;
 	gossip_account_param_get (account,
 				  "password", &account_password,
-				  "id", &account_id,
 				  "resource", &account_resource,
 				  NULL);
+	account_id = gossip_account_get_id (account);
 
 	if (!account_password || g_utf8_strlen (account_password, -1) < 1) {
 		gossip_debug (DEBUG_DOMAIN, "Requesting password for:'%s'",
@@ -1136,13 +1151,13 @@ jabber_new_account (GossipProtocol *protocol)
 	/* Set a default value for each account parameter */
 	account = g_object_new (GOSSIP_TYPE_ACCOUNT,
 				"type", GOSSIP_ACCOUNT_TYPE_JABBER,
+				"id", id,
 				"name", _("new account"),
 				"auto_connect", TRUE,
 				"use_proxy", FALSE,
 				NULL);
 
 	gossip_account_param_new (account,
-				  "id", G_TYPE_STRING, id, 0,
 				  "password", G_TYPE_STRING, "", 0,
 				  "resource", G_TYPE_STRING, _("Home"), 0,
 				  "server", G_TYPE_STRING, server, 0,
@@ -1153,6 +1168,35 @@ jabber_new_account (GossipProtocol *protocol)
 	return account;
 }
 
+static void
+jabber_get_avatar_requirements (GossipProtocol  *protocol,
+				guint           *min_width,
+				guint           *min_height,
+				guint           *max_width,
+				guint           *max_height,
+				gsize           *max_size,
+				gchar          **format)
+{
+	if (min_width) {
+		*min_width = 32;
+	}
+	if (min_height) {
+		*min_height = 32;
+	}
+	if (max_width) {
+		*max_width = 96;
+	}
+	if (max_height) {
+		*max_height = 96;
+	}
+	if (max_size) {
+		*max_size = 8*1024; /* 8kb */
+	}
+	if (format) {
+		*format = g_strdup ("png");
+	}
+}
+
 static void
 jabber_register_connection_open_cb (LmConnection *connection,
 				    gboolean      result,
@@ -1204,10 +1248,11 @@ jabber_register_connection_open_cb (LmConnection *connection,
 
 	gossip_account_param_get (priv->account,
 				  "server", &server,
-				  "id", &id,
 				  "password", &password,
 				  NULL);
 
+	id = gossip_account_get_id (priv->account);
+
 	m = lm_message_new_with_sub_type (server,
 					  LM_MESSAGE_TYPE_IQ,
 					  LM_MESSAGE_SUB_TYPE_SET);
@@ -2999,11 +3044,13 @@ jabber_chatroom_init (GossipChatroomProviderIface *iface)
 	iface->change_topic    = jabber_chatroom_change_topic;
 	iface->change_nick     = jabber_chatroom_change_nick;
 	iface->leave           = jabber_chatroom_leave;
+	iface->find_by_id      = jabber_chatroom_find_by_id;
 	iface->find            = jabber_chatroom_find;
 	iface->invite          = jabber_chatroom_invite;
 	iface->invite_accept   = jabber_chatroom_invite_accept;
 	iface->invite_decline  = jabber_chatroom_invite_decline;
 	iface->get_rooms       = jabber_chatroom_get_rooms;
+	iface->browse_rooms    = jabber_chatroom_browse_rooms;
 }
 
 static GossipChatroomId
@@ -3105,8 +3152,8 @@ jabber_chatroom_leave (GossipChatroomProvider *provider,
 }
 
 static GossipChatroom *
-jabber_chatroom_find (GossipChatroomProvider *provider,
-		      GossipChatroomId        id)
+jabber_chatroom_find_by_id (GossipChatroomProvider *provider,
+			    GossipChatroomId        id)
 {
 	GossipJabber     *jabber;
 	GossipJabberPriv *priv;
@@ -3116,7 +3163,23 @@ jabber_chatroom_find (GossipChatroomProvider *provider,
 	jabber = GOSSIP_JABBER (provider);
 	priv = GET_PRIV (jabber);
 
-	return gossip_jabber_chatrooms_find (priv->chatrooms, id);
+	return gossip_jabber_chatrooms_find_by_id (priv->chatrooms, id);
+}
+
+static GossipChatroom *
+jabber_chatroom_find (GossipChatroomProvider *provider,
+		      GossipChatroom         *chatroom)
+{
+	GossipJabber     *jabber;
+	GossipJabberPriv *priv;
+
+	g_return_val_if_fail (GOSSIP_IS_JABBER (provider), NULL);
+	g_return_val_if_fail (GOSSIP_IS_CHATROOM (chatroom), NULL);
+
+	jabber = GOSSIP_JABBER (provider);
+	priv = GET_PRIV (jabber);
+
+	return gossip_jabber_chatrooms_find (priv->chatrooms, chatroom);
 }
 
 static void
@@ -3189,6 +3252,26 @@ jabber_chatroom_get_rooms (GossipChatroomProvider *provider)
 	return gossip_jabber_chatrooms_get_rooms (priv->chatrooms);
 }
 
+static void
+jabber_chatroom_browse_rooms (GossipChatroomProvider *provider,
+			      const gchar            *server,
+			      GossipChatroomBrowseCb  callback,
+			      gpointer                user_data)
+{
+	GossipJabber     *jabber;
+	GossipJabberPriv *priv;
+
+	g_return_if_fail (GOSSIP_IS_JABBER (provider));
+	g_return_if_fail (server != NULL);
+	g_return_if_fail (callback != NULL);
+
+	jabber = GOSSIP_JABBER (provider);
+	priv = GET_PRIV (jabber);
+
+	gossip_jabber_chatrooms_browse_rooms (priv->chatrooms, server, 
+					      callback, user_data);
+}
+
 /*
  * ft
  */
