@@ -268,7 +268,7 @@ gossip_chat_view_class_init (GossipChatViewClass *klass)
 
 	object_class->finalize = chat_view_finalize;
 	widget_class->size_allocate = chat_view_size_allocate;
-	widget_class->drag_motion = chat_view_drag_motion;
+ 	widget_class->drag_motion = chat_view_drag_motion; 
 
 	g_type_class_add_private (object_class, sizeof (GossipChatViewPriv));
 }
@@ -363,7 +363,7 @@ chat_view_drag_motion (GtkWidget        *widget,
 	 * the result of dragging something across it.
 	 */
 
-	return TRUE;
+	return FALSE;
 }
 
 static void
@@ -532,7 +532,7 @@ chat_view_populate_popup (GossipChatView *view,
 						&start, &end, FALSE);
 	}
 
-	if (!str || strlen (str) < 1) {
+	if (G_STR_EMPTY (str)) {
 		g_free (str);
 		return;
 	}
@@ -1423,7 +1423,12 @@ chat_view_invite_join_cb (GossipChatroomProvider   *provider,
 			  gint                      id,
 			  gpointer                  user_data)
 {
-	gossip_group_chat_new (provider, id);
+	GossipChatroom  *chatroom;
+	GossipGroupChat *chat;
+
+	chatroom = gossip_chatroom_provider_find_by_id (provider, id);
+	chat = gossip_group_chat_new (provider, chatroom);
+/* 	g_object_unref (chat); */
 }
 
 static void
@@ -1663,12 +1668,12 @@ gossip_chat_view_append_invite (GossipChatView *view,
 	reason = gossip_chatroom_invite_get_reason (invite);
 
 	s = g_string_new ("");
-	if (body && strlen (body) > 0) {
+	if (!G_STR_EMPTY (body)) {
 		s = g_string_append (s, body);
 	}
 
 	/* Make sure the reason is not the body (or in the body) */
-	if (reason && strlen (reason) > 0 && !strstr (body, reason)) {
+	if (!G_STR_EMPTY (reason) && !strstr (body, reason)) {
 		if (s->len > 0) {
 			s = g_string_append_c (s, '\n');
 		}
@@ -1761,6 +1766,79 @@ gossip_chat_view_append_invite (GossipChatView *view,
 	priv->last_block_type = BLOCK_TYPE_INVITE;
 }
 
+void
+gossip_chat_view_append_button (GossipChatView *view,
+				const gchar    *message,
+				GtkWidget      *button1,
+				GtkWidget      *button2)
+{
+	GossipChatViewPriv   *priv;
+	GtkTextChildAnchor   *anchor;
+	GtkTextIter           iter;
+	gboolean              bottom;
+	const gchar          *tag;
+
+	g_return_if_fail (button1 != NULL);
+
+	priv = GET_PRIV (view);
+
+	if (priv->irc_style) {
+		tag = "irc-invite";
+	} else {
+		tag = "fancy-invite";
+	}
+
+	bottom = chat_view_is_scrolled_down (view);
+
+	chat_view_maybe_append_date_and_time (view, NULL);
+
+	if (message) {
+		chat_view_append_text (view, message, tag);
+	}
+
+	gtk_text_buffer_get_end_iter (priv->buffer, &iter);
+
+	anchor = gtk_text_buffer_create_child_anchor (priv->buffer, &iter);
+	gtk_text_view_add_child_at_anchor (GTK_TEXT_VIEW (view), button1, anchor);
+	gtk_widget_show (button1);
+
+	gtk_text_buffer_insert_with_tags_by_name (priv->buffer,
+						  &iter,
+						  " ",
+						  1,
+						  tag,
+						  NULL);
+
+	if (button2) {
+		gtk_text_buffer_get_end_iter (priv->buffer, &iter);
+		
+		anchor = gtk_text_buffer_create_child_anchor (priv->buffer, &iter);
+		gtk_text_view_add_child_at_anchor (GTK_TEXT_VIEW (view), button2, anchor);
+		gtk_widget_show (button2);
+		
+		gtk_text_buffer_insert_with_tags_by_name (priv->buffer,
+							  &iter,
+							  " ",
+							  1,
+							  tag,
+							  NULL);
+	}
+
+	gtk_text_buffer_get_end_iter (priv->buffer, &iter);
+	gtk_text_buffer_insert_with_tags_by_name (priv->buffer,
+						  &iter,
+						  "\n\n",
+						  2,
+						  tag,
+						  NULL);
+
+	if (bottom) {
+		gossip_chat_view_scroll_down (view);
+	}
+
+	priv->last_block_type = BLOCK_TYPE_INVITE;
+}
+
 void
 gossip_chat_view_scroll (GossipChatView *view,
 			 gboolean        allow_scrolling)
@@ -1864,7 +1942,7 @@ gossip_chat_view_find_previous (GossipChatView *view,
 
 	buffer = priv->buffer;
 
-	if (strlen (search_criteria) == 0) {
+	if (G_STR_EMPTY (search_criteria)) {
 		if (priv->find_mark_previous) {
 			gtk_text_buffer_get_start_iter (buffer, &iter_at_mark);
 
@@ -1979,7 +2057,7 @@ gossip_chat_view_find_next (GossipChatView *view,
 
 	buffer = priv->buffer;
 
-	if (strlen (search_criteria) == 0) {
+	if (G_STR_EMPTY (search_criteria)) {
 		if (priv->find_mark_next) {
 			gtk_text_buffer_get_start_iter (buffer, &iter_at_mark);
 
@@ -2151,7 +2229,7 @@ gossip_chat_view_highlight (GossipChatView *view,
 					    &iter_start,
 					    &iter_end);
 
-	if (!text || strlen (text) < 1) {
+	if (G_STR_EMPTY (text)) {
 		return;
 	}
 
