@@ -67,13 +67,13 @@ typedef enum {
 	GOSSIP_SMILEY_OOOH,         /*  :x   */
 	GOSSIP_SMILEY_LOOKAWAY,     /*  *)   */
 	GOSSIP_SMILEY_BLUSH,        /*  *S   */
-	GOSSIP_SMILEY_COOLBIGSMILE, /*  8D   */ 
+	GOSSIP_SMILEY_COOLBIGSMILE, /*  8D   */
 	GOSSIP_SMILEY_ANGRY,        /*  :@   */
 	GOSSIP_SMILEY_BOSS,         /*  @)   */
 	GOSSIP_SMILEY_MONKEY,       /*  #)   */
 	GOSSIP_SMILEY_SILLY,        /*  O)   */
 	GOSSIP_SMILEY_SICK,         /*  +o(  */
-	
+
 	GOSSIP_SMILEY_COUNT
 } GossipSmiley;
 
@@ -91,23 +91,32 @@ void            gossip_chat_view_append_event              (GossipChatView *view
 							    const gchar    *str);
 void            gossip_chat_view_append_invite             (GossipChatView *view,
 							    GossipMessage  *message);
-/* void            gossip_chat_view_set_margin                (GossipChatView *view, */
-/* 							    gint            margin); */
+void            gossip_chat_view_append_button             (GossipChatView *view,
+							    const gchar    *message,
+							    GtkWidget      *button1,
+							    GtkWidget      *button2);
+void            gossip_chat_view_set_margin                (GossipChatView *view,
+							    gint            margin);
 void            gossip_chat_view_scroll                    (GossipChatView *view,
 							    gboolean        allow_scrolling);
 void            gossip_chat_view_scroll_down               (GossipChatView *view);
-
 gboolean        gossip_chat_view_get_selection_bounds      (GossipChatView *view,
 							    GtkTextIter    *start,
 							    GtkTextIter    *end);
 void            gossip_chat_view_clear                     (GossipChatView *view);
-void            gossip_chat_view_find                      (GossipChatView *view, 
+gboolean        gossip_chat_view_find_previous             (GossipChatView *view,
 							    const gchar    *search_criteria,
 							    gboolean        new_search);
-void            gossip_chat_view_highlight                 (GossipChatView *view, 
+gboolean        gossip_chat_view_find_next                 (GossipChatView *view,
+							    const gchar    *search_criteria,
+							    gboolean        new_search);
+void            gossip_chat_view_find_abilities            (GossipChatView *view,
+							    const gchar    *search_criteria,
+							    gboolean       *can_do_previous,
+							    gboolean       *can_do_next);
+void            gossip_chat_view_highlight                 (GossipChatView *view,
 							    const gchar    *text);
 void            gossip_chat_view_copy_clipboard            (GossipChatView *view);
-
 gboolean        gossip_chat_view_get_irc_style             (GossipChatView *view);
 void            gossip_chat_view_set_irc_style             (GossipChatView *view,
 							    gboolean        irc_style);
@@ -118,4 +127,7 @@ const gchar *   gossip_chat_view_get_smiley_text           (GossipSmiley    smil
 GtkWidget *     gossip_chat_view_get_smiley_menu           (GCallback       callback,
 							    gpointer        user_data,
 							    GtkTooltips    *tooltips);
+void            gossip_chat_view_set_is_group_chat         (GossipChatView *view,
+							    gboolean        is_group_chat);
+
 #endif /* __GOSSIP_CHAT_VIEW_H__ */
