@@ -30,6 +30,7 @@
 
 #include <libgossip/gossip-debug.h>
 #include <libgossip/gossip-conf.h>
+#include <libgossip/gossip-utils.h>
 
 #include "gossip-app.h"
 #include "gossip-chat.h"
@@ -55,8 +56,8 @@ struct _GossipChatPriv {
 
 	GtkTooltips      *tooltips;
 
-	/* Used to automatically shrink a window that has temporarily grown due
-	 * to long input.
+	/* Used to automatically shrink a window that has temporarily
+	 * grown due to long input. 
 	 */
 	gint              padding_height;
 	gint              default_window_height;
@@ -495,7 +496,7 @@ chat_text_populate_popup_cb (GtkTextView *view,
 						&start, &end, FALSE);
 	}
 
-	if (!str || strlen (str) == 0) {
+	if (G_STR_EMPTY (str)) {
 		return;
 	}
 
@@ -691,6 +692,18 @@ gossip_chat_is_group_chat (GossipChat *chat)
 	return FALSE;
 }
 
+gboolean 
+gossip_chat_is_connected (GossipChat *chat)
+{
+	g_return_val_if_fail (GOSSIP_IS_CHAT (chat), FALSE);
+
+	if (GOSSIP_CHAT_GET_CLASS (chat)->is_connected) {
+		return GOSSIP_CHAT_GET_CLASS (chat)->is_connected (chat);
+	}
+
+	return FALSE;
+}
+
 gboolean
 gossip_chat_get_show_contacts (GossipChat *chat)
 {
