@@ -48,9 +48,10 @@ struct _GossipGroupChatClass {
 
 GType            gossip_group_chat_get_type              (void) G_GNUC_CONST;
 GossipGroupChat *gossip_group_chat_new                   (GossipChatroomProvider *provider,
-							  GossipChatroomId        id);
+							  GossipChatroom         *chatroom);
 GossipChatroomId gossip_group_chat_get_chatroom_id       (GossipGroupChat        *groupchat);
 GossipChatroomProvider *
 		 gossip_group_chat_get_chatroom_provider (GossipGroupChat        *groupchat);
+GossipChatroom * gossip_group_chat_get_chatroom          (GossipGroupChat        *groupchat);
 
 #endif /* __GOSSIP_GROUP_CHAT_H__ */
