@@ -652,7 +652,7 @@ log_get_filename_by_date_for_contact (GossipContact *contact,
 		return NULL;
 	}
 
-	gossip_account_param_get (account, "id", &account_id, NULL);
+	account_id = gossip_account_get_id (account);
 	account_id_escaped = log_escape (account_id);
 
 	log_check_dir (&log_directory);
@@ -718,7 +718,7 @@ log_get_filename_by_date_for_chatroom (GossipChatroom *chatroom,
 		return NULL;
 	}
 
-	gossip_account_param_get (account, "id", &account_id, NULL);
+	account_id = gossip_account_get_id (account);
 	account_id_escaped = log_escape (account_id);
 
 	log_check_dir (&log_directory);
@@ -777,7 +777,7 @@ log_get_contacts_filename (GossipAccount *account)
 		return filename;
 	}
 
-	gossip_account_param_get (account, "id", &account_id, NULL);
+	account_id = gossip_account_get_id (account);
 	gossip_debug (DEBUG_DOMAIN, 
 		      "No contacts file recorded against account id:'%s'...",
 		      account_id);
@@ -904,7 +904,7 @@ gossip_log_get_own_contact (GossipAccount *account)
 
 	g_key_file_free (key_file);
 
-	gossip_account_param_get (account, "id", &id, NULL);
+	id = gossip_account_get_id (account);
 
 	if (!name) {
 		gossip_debug (DEBUG_DOMAIN, "No name in contacts file, using id for name.");
@@ -988,7 +988,7 @@ gossip_log_get_contacts (GossipAccount *account)
 		return NULL;
 	}
 
-	gossip_account_param_get (account, "id", &account_id, NULL);
+	account_id = gossip_account_get_id (account);
 	account_id_escaped = log_escape (account_id);
 
 	directory = g_build_path (G_DIR_SEPARATOR_S,
@@ -1055,7 +1055,7 @@ gossip_log_get_chatrooms (GossipAccount *account)
 		return NULL;
 	}
 
-	gossip_account_param_get (account, "id", &account_id, NULL);
+	account_id = gossip_account_get_id (account);
 	account_id_escaped = log_escape (account_id);
 
 	directory = g_build_path (G_DIR_SEPARATOR_S,
@@ -1143,7 +1143,7 @@ gossip_log_get_dates_for_contact (GossipContact *contact)
 		return NULL;
 	}
 
-	gossip_account_param_get (account, "id", &account_id, NULL);
+	account_id = gossip_account_get_id (account);
 	account_id_escaped = log_escape (account_id);
 
 	contact_id = gossip_contact_get_id (contact);
@@ -1577,7 +1577,7 @@ gossip_log_get_dates_for_chatroom (GossipChatroom *chatroom)
 		return NULL;
 	}
 
-	gossip_account_param_get (account, "id", &account_id, NULL);
+	account_id = gossip_account_get_id (account);
 	account_id_escaped = log_escape (account_id);
 
 	chatroom_id = gossip_chatroom_get_id_str (chatroom);
@@ -1906,8 +1906,7 @@ gossip_log_search_new (const gchar *text)
 	gchar                *contents_casefold;
 	GList                *hits = NULL;
 
-	g_return_val_if_fail (text != NULL, NULL);
-	g_return_val_if_fail (strlen (text) > 0, NULL);
+	g_return_val_if_fail (!G_STR_EMPTY (text), NULL);
 
 	text_casefold = g_utf8_casefold (text, -1);
 
