@@ -21,12 +21,10 @@
 
 #include <config.h>
 #include <string.h>
-#include <gconf/gconf-client.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
 #include <glib/gi18n.h>
-#include <libgnomeui/gnome-href.h>
 
 #include <libgossip/gossip-debug.h>
 #include <libgossip/gossip-message.h>
@@ -42,18 +40,17 @@
 #include "gossip-stock.h"
 #include "gossip-ui-utils.h"
 
-#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), \
-		       GOSSIP_TYPE_PRIVATE_CHAT, GossipPrivateChatPriv))
-
-#define DEBUG_DOMAIN "PrivateChat"
+#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_PRIVATE_CHAT, GossipPrivateChatPriv))
 
 #define IS_ENTER(v) (v == GDK_Return || v == GDK_ISO_Enter || v == GDK_KP_Enter)
 
+#define DEBUG_DOMAIN "PrivateChat"
+
 #define COMPOSING_STOP_TIMEOUT 5
 
 struct _GossipPrivateChatPriv {
-        GossipContact    *contact;
-        GossipContact    *own_contact;
+	GossipContact    *contact;
+	GossipContact    *own_contact;
 	gchar            *name;
 
 	GdkPixbuf        *own_avatar;
@@ -62,12 +59,12 @@ struct _GossipPrivateChatPriv {
 	gchar            *locked_resource;
 	gchar            *roster_resource;
 
-        guint             composing_stop_timeout_id;
+	guint             composing_stop_timeout_id;
 	guint             scroll_idle_id;
 
 	gboolean          is_online;
 
-        GtkWidget        *widget;
+	GtkWidget        *widget;
 	GtkWidget	 *text_view_sw;
 };
 
@@ -97,6 +94,7 @@ static void           private_chat_protocol_connected_cb        (GossipSession
 static void           private_chat_protocol_disconnected_cb     (GossipSession          *session,
 								 GossipAccount          *account,
 								 GossipProtocol         *protocol,
+								 gint                    reason,
 								 GossipPrivateChat      *chat);
 static void           private_chat_composing_cb                 (GossipSession          *session,
 								 GossipContact          *contact,
@@ -110,8 +108,7 @@ static void           private_chat_input_text_buffer_changed_cb (GtkTextBuffer
 static gboolean       private_chat_text_view_focus_in_event_cb  (GtkWidget              *widget,
 								 GdkEvent               *event,
 								 GossipPrivateChat      *chat);
-static gboolean       private_chat_delete_event_cb              (GtkWidget              *widget,
-								 GdkEvent               *event,
+static void           private_chat_widget_destroy_cb            (GtkWidget              *widget,
 								 GossipPrivateChat      *chat);
 static void           private_chat_own_avatar_notify_cb         (GossipContact          *contact,
 								 GParamSpec             *pspec,
@@ -122,10 +119,11 @@ static void           private_chat_other_avatar_notify_cb       (GossipContact
 static const gchar *  private_chat_get_name                     (GossipChat             *chat);
 static gchar *        private_chat_get_tooltip                  (GossipChat             *chat);
 static GdkPixbuf *    private_chat_get_status_pixbuf            (GossipChat             *chat);
-static GtkWidget *    private_chat_get_widget                   (GossipChat             *chat);
 static GossipContact *private_chat_get_contact                  (GossipChat             *chat);
 static GossipContact *private_chat_get_own_contact              (GossipChat             *chat);
-static GdkPixbuf *    private_chat_pad_to_size                  (GdkPixbuf              *pixbuf, 
+static GtkWidget *    private_chat_get_widget                   (GossipChat             *chat);
+static gboolean       private_chat_is_connected                 (GossipChat             *chat);
+static GdkPixbuf *    private_chat_pad_to_size                  (GdkPixbuf              *pixbuf,
 								 gint                    width,
 								 gint                    height,
 								 gint                    extra_padding_right);
@@ -135,17 +133,21 @@ G_DEFINE_TYPE (GossipPrivateChat, gossip_private_chat, GOSSIP_TYPE_CHAT);
 static void
 gossip_private_chat_class_init (GossipPrivateChatClass *klass)
 {
-        GObjectClass    *object_class = G_OBJECT_CLASS (klass);
+	GObjectClass    *object_class = G_OBJECT_CLASS (klass);
 	GossipChatClass *chat_class = GOSSIP_CHAT_CLASS (klass);
 
-        object_class->finalize = private_chat_finalize;
-	
+	object_class->finalize = private_chat_finalize;
+
 	chat_class->get_name          = private_chat_get_name;
 	chat_class->get_tooltip       = private_chat_get_tooltip;
 	chat_class->get_status_pixbuf = private_chat_get_status_pixbuf;
 	chat_class->get_contact       = private_chat_get_contact;
 	chat_class->get_own_contact   = private_chat_get_own_contact;
 	chat_class->get_widget        = private_chat_get_widget;
+	chat_class->get_show_contacts = NULL;
+	chat_class->set_show_contacts = NULL;
+	chat_class->is_group_chat     = NULL;
+	chat_class->is_connected      = private_chat_is_connected;
 
 	g_type_class_add_private (object_class, sizeof (GossipPrivateChatPriv));
 }
@@ -153,13 +155,13 @@ gossip_private_chat_class_init (GossipPrivateChatClass *klass)
 static void
 gossip_private_chat_init (GossipPrivateChat *chat)
 {
-        GossipPrivateChatPriv *priv;
-	
+	GossipPrivateChatPriv *priv;
+
 	priv = GET_PRIV (chat);
 
 	priv->is_online = FALSE;
 
-        private_chat_create_ui (chat);
+	private_chat_create_ui (chat);
 
 	gossip_debug (DEBUG_DOMAIN, "Connecting");
 
@@ -172,7 +174,7 @@ gossip_private_chat_init (GossipPrivateChat *chat)
 				 "protocol-disconnected",
 				 G_CALLBACK (private_chat_protocol_disconnected_cb),
 				 chat, 0);
-	
+
 	g_signal_connect_object (gossip_app_get_session (),
 				 "composing",
 				 G_CALLBACK (private_chat_composing_cb),
@@ -182,12 +184,22 @@ gossip_private_chat_init (GossipPrivateChat *chat)
 static void
 private_chat_finalize (GObject *object)
 {
-        GossipPrivateChat     *chat;
+	GossipPrivateChat     *chat;
 	GossipPrivateChatPriv *priv;
 
+	gossip_debug (DEBUG_DOMAIN, "Finalized:%p", object);
+	
 	chat = GOSSIP_PRIVATE_CHAT (object);
 	priv = GET_PRIV (chat);
 
+	g_signal_handlers_disconnect_by_func (priv->contact,
+					      private_chat_own_avatar_notify_cb,
+					      chat);
+	g_signal_handlers_disconnect_by_func (priv->contact,
+					      private_chat_other_avatar_notify_cb,
+					      chat);
+		
+
 	if (priv->contact) {
 		g_object_unref (priv->contact);
 	}
@@ -211,28 +223,38 @@ private_chat_finalize (GObject *object)
 	g_free (priv->name);
 	g_free (priv->locked_resource);
 	g_free (priv->roster_resource);
-	
+
 	private_chat_composing_remove_timeout (chat);
-	
-        G_OBJECT_CLASS (gossip_private_chat_parent_class)->finalize (object);
+
+	G_OBJECT_CLASS (gossip_private_chat_parent_class)->finalize (object);
 }
 
 static void
 private_chat_create_ui (GossipPrivateChat *chat)
 {
+	GladeXML              *glade;
 	GossipPrivateChatPriv *priv;
-        GtkTextBuffer         *buffer;
+	GtkTextBuffer         *buffer;
 	GtkWidget             *input_text_view_sw;
 
 	priv = GET_PRIV (chat);
 
-        gossip_glade_get_file_simple (GLADEDIR "/chat.glade",
-                                      "chat_widget",
-                                      NULL,
-                                      "chat_widget", &priv->widget,
-                                      "chat_view_sw", &priv->text_view_sw,
+	glade = gossip_glade_get_file ("chat.glade",
+				       "chat_widget",
+				       NULL,
+				      "chat_widget", &priv->widget,
+				      "chat_view_sw", &priv->text_view_sw,
 				      "input_text_view_sw", &input_text_view_sw,
-                                      NULL);
+				       NULL);
+
+	gossip_glade_connect (glade,
+			      chat,
+			      "chat_widget", "destroy", private_chat_widget_destroy_cb,
+			      NULL);
+
+	g_object_unref (glade);
+
+	g_object_set_data (G_OBJECT (priv->widget), "chat", chat);
 
 	gtk_container_add (GTK_CONTAINER (priv->text_view_sw),
 			   GTK_WIDGET (GOSSIP_CHAT (chat)->view));
@@ -242,27 +264,20 @@ private_chat_create_ui (GossipPrivateChat *chat)
 			   GOSSIP_CHAT (chat)->input_text_view);
 	gtk_widget_show (GOSSIP_CHAT (chat)->input_text_view);
 
-	g_object_ref (priv->widget);
-	g_object_set_data (G_OBJECT (priv->widget), "chat", chat);
-
 	g_signal_connect (GOSSIP_CHAT (chat)->input_text_view,
 			  "key_press_event",
 			  G_CALLBACK (private_chat_input_key_press_event_cb),
 			  chat);
 
-        buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (GOSSIP_CHAT (chat)->input_text_view));
-        g_signal_connect (buffer,
-                          "changed",
-                          G_CALLBACK (private_chat_input_text_buffer_changed_cb),
-                          chat);
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (GOSSIP_CHAT (chat)->input_text_view));
+	g_signal_connect (buffer,
+			  "changed",
+			  G_CALLBACK (private_chat_input_text_buffer_changed_cb),
+			  chat);
 	g_signal_connect (GOSSIP_CHAT (chat)->view,
 			  "focus_in_event",
 			  G_CALLBACK (private_chat_text_view_focus_in_event_cb),
 			  chat);
-	g_signal_connect (priv->widget,
-			  "delete_event",
-			  G_CALLBACK (private_chat_delete_event_cb),
-			  NULL);
 
 	gtk_widget_grab_focus (GOSSIP_CHAT (chat)->input_text_view);
 }
@@ -285,7 +300,7 @@ private_chat_update_locked_resource (GossipPrivateChat *chat)
 		return;
 	}
 
-	roster_resource = gossip_session_get_active_resource (gossip_app_get_session (), 
+	roster_resource = gossip_session_get_active_resource (gossip_app_get_session (),
 							      priv->contact);
 
 	/* It seems like some agents don't set a resource sometimes (ICQ for
@@ -300,7 +315,7 @@ private_chat_update_locked_resource (GossipPrivateChat *chat)
 		priv->locked_resource = NULL;
 		return;
 	}
-	
+
 	if (priv->roster_resource &&
 	    g_ascii_strcasecmp (priv->roster_resource, roster_resource) == 0) {
 		gossip_debug (DEBUG_DOMAIN, "Roster unchanged");
@@ -308,12 +323,12 @@ private_chat_update_locked_resource (GossipPrivateChat *chat)
 		if (!priv->locked_resource) {
 			priv->locked_resource = g_strdup (roster_resource);
 		}
-		
+
 		return;
 	}
-	
+
 	gossip_debug (DEBUG_DOMAIN, "New roster resource: %s", roster_resource);
-	
+
 	g_free (priv->roster_resource);
 	priv->roster_resource = g_strdup (roster_resource);
 
@@ -323,66 +338,66 @@ private_chat_update_locked_resource (GossipPrivateChat *chat)
 
 static void
 private_chat_send (GossipPrivateChat *chat,
-                   const gchar       *msg)
+		   const gchar       *msg)
 {
 	GossipPrivateChatPriv *priv;
 	GossipMessage         *message;
 
 	priv = GET_PRIV (chat);
 
-        if (msg == NULL || msg[0] == '\0') {
-                return;
-        }
+	if (msg == NULL || msg[0] == '\0') {
+		return;
+	}
 
-        if (g_ascii_strcasecmp (msg, "/clear") == 0) {
+	if (g_ascii_strcasecmp (msg, "/clear") == 0) {
 		gossip_chat_view_clear (GOSSIP_CHAT (chat)->view);
-                return;
-        }
+		return;
+	}
 
 	gossip_app_set_not_away ();
 
-        private_chat_composing_remove_timeout (chat);
+	private_chat_composing_remove_timeout (chat);
 
 	private_chat_update_locked_resource (chat);
-	
+
 	message = gossip_message_new (GOSSIP_MESSAGE_TYPE_NORMAL, priv->contact);
 
 	if (priv->locked_resource) {
 		gossip_message_set_explicit_resource (message, priv->locked_resource);
 	}
-	
+
 	gossip_message_set_body (message, msg);
 	gossip_message_request_composing (message);
 	gossip_message_set_sender (message, priv->own_contact);
 
 	gossip_log_message_for_contact (message, FALSE);
 
-	gossip_chat_view_append_message_from_self (GOSSIP_CHAT (chat)->view, 
-						   message, 
+	gossip_chat_view_append_message_from_self (GOSSIP_CHAT (chat)->view,
+						   message,
 						   priv->own_contact,
 						   priv->own_avatar);
 
 	gossip_session_send_message (gossip_app_get_session (), message);
-	
+
 	g_object_unref (message);
 }
 
 static void
 private_chat_composing_start (GossipPrivateChat *chat)
 {
-        GossipPrivateChatPriv *priv;
+	GossipPrivateChatPriv *priv;
 
-        priv = GET_PRIV (chat);
+	priv = GET_PRIV (chat);
 
-        if (priv->composing_stop_timeout_id) {
-                /* Just restart the timeout */
-                private_chat_composing_remove_timeout (chat);
-        } else {
-                gossip_session_send_composing (gossip_app_get_session (),
-                                               priv->contact, TRUE);
-        }
+	if (priv->composing_stop_timeout_id) {
+		/* Just restart the timeout */
+		private_chat_composing_remove_timeout (chat);
+	} else {
+		gossip_session_send_composing (gossip_app_get_session (),
+					       priv->contact, TRUE);
+	}
 
-        priv->composing_stop_timeout_id = g_timeout_add (
+	priv->composing_stop_timeout_id = g_timeout_add (
 		1000 * COMPOSING_STOP_TIMEOUT,
 		(GSourceFunc) private_chat_composing_stop_timeout_cb,
 		chat);
@@ -391,13 +406,13 @@ private_chat_composing_start (GossipPrivateChat *chat)
 static void
 private_chat_composing_stop (GossipPrivateChat *chat)
 {
-        GossipPrivateChatPriv *priv;
+	GossipPrivateChatPriv *priv;
 
-        priv = GET_PRIV (chat);
+	priv = GET_PRIV (chat);
 
 	private_chat_composing_remove_timeout (chat);
-        gossip_session_send_composing (gossip_app_get_session (),
-                                       priv->contact, FALSE);
+	gossip_session_send_composing (gossip_app_get_session (),
+				       priv->contact, FALSE);
 }
 
 static void
@@ -407,10 +422,10 @@ private_chat_composing_remove_timeout (GossipPrivateChat *chat)
 
 	priv = GET_PRIV (chat);
 
-        if (priv->composing_stop_timeout_id) {
-                g_source_remove (priv->composing_stop_timeout_id);
-                priv->composing_stop_timeout_id = 0;
-        }
+	if (priv->composing_stop_timeout_id) {
+		g_source_remove (priv->composing_stop_timeout_id);
+		priv->composing_stop_timeout_id = 0;
+	}
 }
 
 static gboolean
@@ -419,18 +434,18 @@ private_chat_composing_stop_timeout_cb (GossipPrivateChat *chat)
 	GossipPrivateChatPriv *priv;
 
 	priv = GET_PRIV (chat);
-	
-        priv->composing_stop_timeout_id = 0;
-        gossip_session_send_composing (gossip_app_get_session (),
-                                       priv->contact, FALSE);
 
-        return FALSE;
+	priv->composing_stop_timeout_id = 0;
+	gossip_session_send_composing (gossip_app_get_session (),
+				       priv->contact, FALSE);
+
+	return FALSE;
 }
 
 static void
 private_chat_contact_presence_updated (gpointer           not_used,
 				       GossipContact     *contact,
-			               GossipPrivateChat *chat)
+				       GossipPrivateChat *chat)
 {
 	GossipPrivateChatPriv *priv;
 
@@ -450,10 +465,10 @@ private_chat_contact_presence_updated (gpointer           not_used,
 
 		if (priv->is_online) {
 			gchar *msg;
-			
+
 			msg = g_strdup_printf (_("%s went offline"),
 					       gossip_contact_get_name (priv->contact));
-			gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view, 
+			gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view,
 						       msg);
 			g_free (msg);
 		}
@@ -461,11 +476,11 @@ private_chat_contact_presence_updated (gpointer           not_used,
 		priv->is_online = FALSE;
 
 		g_signal_emit_by_name (chat, "composing", FALSE);
-				
+
 	} else {
 		if (!priv->is_online) {
 			gchar *msg;
-		
+
 			msg = g_strdup_printf (_("%s has come online"),
 					       gossip_contact_get_name (priv->contact));
 			gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view,
@@ -479,8 +494,8 @@ private_chat_contact_presence_updated (gpointer           not_used,
 
 static void
 private_chat_contact_updated (gpointer           not_used,
-		              GossipContact     *contact, 
-		              GossipPrivateChat *chat)
+			      GossipContact     *contact,
+			      GossipPrivateChat *chat)
 {
 	GossipPrivateChatPriv *priv;
 
@@ -499,13 +514,13 @@ private_chat_contact_updated (gpointer           not_used,
 
 static void
 private_chat_contact_added (gpointer           not_user,
-		            GossipContact     *contact,
-		            GossipPrivateChat *chat)
+			    GossipContact     *contact,
+			    GossipPrivateChat *chat)
 {
 	GossipPrivateChatPriv *priv;
 
 	priv = GET_PRIV (chat);
-	
+
 	if (!gossip_contact_equal (contact, priv->contact)) {
 		return;
 	}
@@ -513,7 +528,7 @@ private_chat_contact_added (gpointer           not_user,
 	g_signal_handlers_disconnect_by_func (priv->contact,
 					      private_chat_other_avatar_notify_cb,
 					      chat);
-		
+
 	g_object_unref (priv->contact);
 	priv->contact = g_object_ref (contact);
 
@@ -541,13 +556,18 @@ private_chat_protocol_connected_cb (GossipSession     *session,
 
 	gtk_widget_set_sensitive (GOSSIP_CHAT (chat)->input_text_view, TRUE);
 
-	gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view, _("Connected"));
+	/* i18n: An event, as in "has now been connected". */
+	gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view,
+				       _("Connected"));
+
+	g_signal_emit_by_name (chat, "status-changed");
 }
 
 static void
 private_chat_protocol_disconnected_cb (GossipSession     *session,
 				       GossipAccount     *account,
 				       GossipProtocol    *protocol,
+				       gint               reason,
 				       GossipPrivateChat *chat)
 {
 	GossipPrivateChatPriv *priv;
@@ -562,7 +582,9 @@ private_chat_protocol_disconnected_cb (GossipSession     *session,
 
 	gtk_widget_set_sensitive (GOSSIP_CHAT (chat)->input_text_view, FALSE);
 
-	gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view, _("Disconnected"));
+	/* i18n: An event, as in "has now been disconnected". */
+	gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view,
+				       _("Disconnected"));
 
 	g_signal_emit_by_name (chat, "status-changed");
 
@@ -617,24 +639,24 @@ private_chat_input_text_view_send (GossipPrivateChat *chat)
 
 static gboolean
 private_chat_input_key_press_event_cb (GtkWidget         *widget,
-                                       GdkEventKey       *event,
-                                       GossipPrivateChat *chat)
+				       GdkEventKey       *event,
+				       GossipPrivateChat *chat)
 {
 	GossipPrivateChatPriv *priv;
 	GtkAdjustment         *adj;
 	gdouble                val;
-	
+
 	priv = GET_PRIV (chat);
 
 	if (event->keyval == GDK_Tab && !(event->state & GDK_CONTROL_MASK)) {
 		return TRUE;
 	}
-	
+
 	/* Catch enter but not ctrl/shift-enter */
 	if (IS_ENTER (event->keyval) && !(event->state & GDK_SHIFT_MASK)) {
 		GtkTextView *view;
 
-		/* This is to make sure that kinput2 gets the enter. And if 
+		/* This is to make sure that kinput2 gets the enter. And if
 		 * it's handled there we shouldn't send on it. This is because
 		 * kinput2 uses Enter to commit letters. See:
 		 * http://bugzilla.redhat.com/bugzilla/show_bug.cgi?id=104299
@@ -645,11 +667,11 @@ private_chat_input_key_press_event_cb (GtkWidget         *widget,
 			GTK_TEXT_VIEW (GOSSIP_CHAT (chat)->input_text_view)->need_im_reset = TRUE;
 			return TRUE;
 		}
-		
+
 		private_chat_input_text_view_send (chat);
 		return TRUE;
 	}
-	
+
 	if (IS_ENTER (event->keyval) && (event->state & GDK_SHIFT_MASK)) {
 		/* Newline for shift-enter. */
 		return FALSE;
@@ -658,24 +680,24 @@ private_chat_input_key_press_event_cb (GtkWidget         *widget,
 		 event->keyval == GDK_Page_Up) {
 		adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (priv->text_view_sw));
 		gtk_adjustment_set_value (adj, adj->value - adj->page_size);
-		
+
 		return TRUE;
 	}
 	else if ((event->state & GDK_CONTROL_MASK) != GDK_CONTROL_MASK &&
 		 event->keyval == GDK_Page_Down) {
 		adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (priv->text_view_sw));
-		val = MIN (adj->value + adj->page_size, adj->upper - adj->page_size);  
+		val = MIN (adj->value + adj->page_size, adj->upper - adj->page_size);
 		gtk_adjustment_set_value (adj, val);
 
 		return TRUE;
 	}
-		
+
 	return FALSE;
 }
 
 static void
-private_chat_input_text_buffer_changed_cb (GtkTextBuffer     *buffer, 
-		                           GossipPrivateChat *chat)
+private_chat_input_text_buffer_changed_cb (GtkTextBuffer     *buffer,
+					   GossipPrivateChat *chat)
 {
 	GossipPrivateChatPriv *priv;
 
@@ -690,20 +712,21 @@ private_chat_input_text_buffer_changed_cb (GtkTextBuffer     *buffer,
 
 static gboolean
 private_chat_text_view_focus_in_event_cb (GtkWidget         *widget,
-				          GdkEvent          *event,
-				          GossipPrivateChat *chat)
+					  GdkEvent          *event,
+					  GossipPrivateChat *chat)
 {
 	gtk_widget_grab_focus (GOSSIP_CHAT (chat)->input_text_view);
-	
+
 	return TRUE;
 }
 
-static gboolean
-private_chat_delete_event_cb (GtkWidget         *widget,
-		              GdkEvent          *event,
-		              GossipPrivateChat *chat)
+static void
+private_chat_widget_destroy_cb (GtkWidget         *widget,
+				GossipPrivateChat *chat)
 {
-	return TRUE;
+	gossip_debug (DEBUG_DOMAIN, "Destroyed");
+
+	g_object_unref (chat);
 }
 
 static void
@@ -715,11 +738,11 @@ private_chat_own_avatar_notify_cb (GossipContact     *contact,
 	GdkPixbuf             *pixbuf;
 
 	priv = GET_PRIV (chat);
-	
+
 	if (priv->own_avatar) {
 		g_object_unref (priv->own_avatar);
 	}
-	
+
 	pixbuf = gossip_pixbuf_avatar_from_contact_scaled (priv->own_contact, 32, 32);
 	if (pixbuf) {
 		priv->own_avatar = private_chat_pad_to_size (pixbuf, 32, 32, 6);
@@ -742,7 +765,7 @@ private_chat_other_avatar_notify_cb (GossipContact     *contact,
 	if (priv->other_avatar) {
 		g_object_unref (priv->other_avatar);
 	}
-	
+
 	pixbuf = gossip_pixbuf_avatar_from_contact_scaled (priv->contact, 32, 32);
 	if (pixbuf) {
 		priv->other_avatar = private_chat_pad_to_size (pixbuf, 32, 32, 6);
@@ -763,7 +786,7 @@ private_chat_get_name (GossipChat *chat)
 
 	return priv->name;
 }
-		
+
 static gchar *
 private_chat_get_tooltip (GossipChat *chat)
 {
@@ -783,7 +806,7 @@ private_chat_get_tooltip (GossipChat *chat)
 				status);
 }
 
-GdkPixbuf *
+static GdkPixbuf *
 private_chat_get_status_pixbuf (GossipChat *chat)
 {
 	GossipPrivateChatPriv *priv;
@@ -832,6 +855,28 @@ private_chat_get_widget (GossipChat *chat)
 	return priv->widget;
 }
 
+static gboolean
+private_chat_is_connected (GossipChat *chat)
+{
+	GossipPrivateChatPriv *priv;
+	GossipAccount         *account;
+
+	g_return_val_if_fail (GOSSIP_IS_PRIVATE_CHAT (chat), FALSE);
+	
+	priv = GET_PRIV (chat);
+
+	if (!priv->own_contact) {
+		return FALSE;
+	}
+	
+	account = gossip_contact_get_account (priv->own_contact);
+	if (!account) {
+		return FALSE;
+	}
+
+	return gossip_session_is_connected (gossip_app_get_session (), account);
+}
+
 /* Scroll down after the back-log has been received. */
 static gboolean
 private_chat_scroll_down_idle_func (GossipChat *chat)
@@ -841,10 +886,10 @@ private_chat_scroll_down_idle_func (GossipChat *chat)
 	priv = GET_PRIV (chat);
 
 	gossip_chat_scroll_down (chat);
- 	g_object_unref (chat);
-	
+	g_object_unref (chat);
+
 	priv->scroll_idle_id = 0;
-	
+
 	return FALSE;
 }
 
@@ -876,7 +921,7 @@ gossip_private_chat_new (GossipContact *own_contact,
 			  "notify::avatar",
 			  G_CALLBACK (private_chat_own_avatar_notify_cb),
 			  chat);
-	
+
 	g_signal_connect (priv->contact,
 			  "notify::avatar",
 			  G_CALLBACK (private_chat_other_avatar_notify_cb),
@@ -884,7 +929,7 @@ gossip_private_chat_new (GossipContact *own_contact,
 
 	private_chat_own_avatar_notify_cb (priv->own_contact, NULL, chat);
 	private_chat_other_avatar_notify_cb (priv->contact, NULL, chat);
-	
+
 	view = GOSSIP_CHAT (chat)->view;
 
 	/* Turn off scrolling temporarily */
@@ -903,28 +948,30 @@ gossip_private_chat_new (GossipContact *own_contact,
 
 		sender = gossip_message_get_sender (message);
 		if (gossip_contact_equal (priv->own_contact, sender)) {
-			gossip_chat_view_append_message_from_self (view, 
+			gossip_chat_view_append_message_from_self (view,
 								   message,
 								   priv->own_contact,
 								   priv->own_avatar);
 		} else {
-			gossip_chat_view_append_message_from_other (view, 
+			gossip_chat_view_append_message_from_other (view,
 								    message,
 								    sender,
 								    priv->other_avatar);
 		}
 	}
 
- 	g_list_foreach (messages, (GFunc) g_object_unref, NULL);
+	g_list_foreach (messages, (GFunc) g_object_unref, NULL);
 	g_list_free (messages);
 
 	/* Turn back on scrolling */
 	gossip_chat_view_scroll (view, TRUE);
 
-	/* Scroll to the most recent messages */
-	g_object_ref (chat);
-	priv->scroll_idle_id = g_idle_add (
-		(GSourceFunc) private_chat_scroll_down_idle_func, chat);
+	/* Scroll to the most recent messages, we reference the chat
+	 * for the duration of the scroll func.
+	 */
+	priv->scroll_idle_id = g_idle_add ((GSourceFunc) 
+					   private_chat_scroll_down_idle_func, 
+					   g_object_ref (chat));
 
 	/* Set up signals */
 	g_signal_connect_object (gossip_app_get_session (),
@@ -975,7 +1022,7 @@ gossip_private_chat_get_history (GossipPrivateChat *chat, gint lines)
 }
 
 void
-gossip_private_chat_append_message (GossipPrivateChat *chat, 
+gossip_private_chat_append_message (GossipPrivateChat *chat,
 				    GossipMessage     *message)
 {
 	GossipPrivateChatPriv *priv;
@@ -983,12 +1030,12 @@ gossip_private_chat_append_message (GossipPrivateChat *chat,
 	GossipChatroomInvite  *invite;
 	const gchar           *resource;
 	const gchar           *subject;
-	
-        g_return_if_fail (GOSSIP_IS_PRIVATE_CHAT (chat));
-        g_return_if_fail (GOSSIP_IS_MESSAGE (message));
+
+	g_return_if_fail (GOSSIP_IS_PRIVATE_CHAT (chat));
+	g_return_if_fail (GOSSIP_IS_MESSAGE (message));
 
 	priv = GET_PRIV (chat);
-	
+
 	gossip_debug (DEBUG_DOMAIN, "Appending message ('%s')",
 		   gossip_contact_get_name (gossip_message_get_sender (message)));
 
@@ -1016,7 +1063,7 @@ gossip_private_chat_append_message (GossipPrivateChat *chat,
 	gossip_log_message_for_contact (message, TRUE);
 
 	subject = gossip_message_get_subject (message);
- 	if (subject) {
+	if (subject) {
 		gchar *str;
 
 		str = g_strdup_printf (_("Subject: %s"), subject);
@@ -1046,7 +1093,7 @@ gossip_private_chat_append_message (GossipPrivateChat *chat,
  * pixbuf. Returns a new ref.
  */
 static GdkPixbuf *
-private_chat_pad_to_size (GdkPixbuf *pixbuf, 
+private_chat_pad_to_size (GdkPixbuf *pixbuf,
 			  gint       width,
 			  gint       height,
 			  gint       extra_padding_right)
@@ -1057,7 +1104,7 @@ private_chat_pad_to_size (GdkPixbuf *pixbuf,
 
 	src_width = gdk_pixbuf_get_width (pixbuf);
 	src_height = gdk_pixbuf_get_height (pixbuf);
-	
+
 	x_offset = (width - src_width) / 2;
 	y_offset = (height - src_height) / 2;
 
@@ -1076,7 +1123,7 @@ private_chat_pad_to_size (GdkPixbuf *pixbuf,
 			      src_height,
 			      padded,
 			      x_offset, /* dest coords */
-			      y_offset); 
+			      y_offset);
 
 	return padded;
 }
