@@ -242,7 +242,7 @@ gossip_password_dialog_run (GossipAccount *account,
 						     str);
 	g_free (str);
 
-	gossip_account_param_get (account, "id", &id, NULL);
+	id = gossip_account_get_id (account);
 	str = g_strdup_printf (_("Logging in with: %s"), id);
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", str);
 	g_free (str);
@@ -432,6 +432,10 @@ gossip_pixbuf_from_account_type (GossipAccountType type,
 	case GOSSIP_ACCOUNT_TYPE_YAHOO:
 		icon_id = "im-yahoo";
 		break;
+	case GOSSIP_ACCOUNT_TYPE_UNKNOWN:
+		/* FIXME: we should have an artwork for unknown protocols */
+		icon_id = "im-jabber";
+		break;
 	default:
 		g_assert_not_reached ();
 	}
@@ -671,6 +675,8 @@ gossip_window_get_is_visible (GtkWindow *window)
 {
 	gboolean visible;
 
+	g_return_val_if_fail (window != NULL, FALSE);
+
 	g_object_get (window,
 		      "visible", &visible,
 		      NULL);
@@ -687,6 +693,8 @@ gossip_window_present (GtkWindow *window,
 	gboolean on_current;
 	guint32  timestamp;
 
+	g_return_if_fail (window != NULL);
+
 	/* There are three cases: hidden, visible, visible on another
 	 * workspace.
 	 */
@@ -796,7 +804,8 @@ gossip_pixbuf_for_chatroom_status (GossipChatroom *chatroom,
 		stock_id = GOSSIP_STOCK_GROUP_MESSAGE;
 		break;
 	case GOSSIP_CHATROOM_STATUS_ERROR:
-		stock_id = GTK_STOCK_DIALOG_ERROR;
+		/* NOTE: GTK_STOCK_DIALOG_ERROR is too big for menu images */
+		stock_id = GTK_STOCK_DIALOG_QUESTION;
 		break;
 	default:
 	case GOSSIP_CHATROOM_STATUS_INACTIVE:
@@ -1821,3 +1830,16 @@ gossip_request_user_attention (void)
 #endif
 }
 
+
+void
+gossip_toggle_button_set_state_quietly (GtkWidget *widget,
+					GCallback  callback,
+					gpointer   user_data,
+					gboolean   active)
+{
+	g_return_if_fail (GTK_IS_TOGGLE_BUTTON (widget));
+
+	g_signal_handlers_block_by_func (widget, callback, user_data);
+	g_object_set (widget, "active", active, NULL);
+	g_signal_handlers_unblock_by_func (widget, callback, user_data);
+}
